#!/usr/bin/perl
BEGIN {
	require "fatlib.pl";
}

use utf8;
use strict;

my $VER="V.1.008n(nanakochi123456)";
my $tarball="denkiyohouh120311.tar.gz";
my $adminmail='nanami@daiba.cx';
my $adminname='でんき予報速報管理人';
my $date_format="Y-m-d H:i:s";

my $data_update=<<EOM;
<li>2011/8/04 14:30 11:40のデータから95%が検出され、実際に正常に自動メール送信をされ、11:40～11:55の間、及び、13:15～14:30までの間 最大96%台を推移していましたが、14:30の供給対需要が96.9%に達した時点で東北電力側から供給、及び、需要が上方修正され、現在の所その時間帯が最大電力と予想される94.8%以降、少しずつ減ってきている模様です。</li>
EOM

my $engine_update=<<EOM;
<li>2011/12/03 14:50 今更ですが、Windows ガジェットに対応した。</li>
<li>2011/11/16 08:00 96％台では電力使用率オーバーをメール配信しないように修正した。97%台からメール配信をします。サイト上では96％台でも警告メッセージを表示します。</li>
<li>2011/8/08 11:40 0時に再び翌日の見通しが配信されるのを修正</li>
<li>2011/8/05 13:21 頻繁に使用率95%を超えることを見込まれるので、メール送信ポリシー及び初期警告を96%まで引き上げた。</li>
<li>2011/8/04 13:33 小数点1桁まで％表示をするようにした。</li>
<li>2011/7/15 16:00 お知らせメールの一部を変更した</li>
<li>2011/7/14 17:00 サイト作成</li>
EOM

my $tclock_update=<<EOM;
<li>2011/7/14 17:00 東京電力の物を元に作成。</li>
EOM

#------------
use CGI;
use Jcode;
use Nana::Mail;
use Nana::YukiWikiDB;
use Encode;
use HTTP::Lite;
use Digest::MD5 qw(md5_hex);
my $debug;

require "common.pl";

$VER=~s/\(.*//g;

my $body;

my($basehref, $basehost, $basepath)=&getbasehref;
my $cache="./cache";
my $date=&date("Y-m-d");
my $hour=&date("H");
my $logfile="$cache/usage-$date.log";

if($ENV{QUERY_STRING}=~/cmd\=twitter\_proxy/) {
	&twitter_proxy;
	exit;
}

if($ENV{QUERY_STRING} eq "tclock") {
	print <<EOM;
Content-type: text/plain

EOM
	my $header="=========================■ 東北電力電気速報 ■=========================\n";
	print Jcode->new($header)->sjis;
	if(open(R,"cache/sokuhou.txt")) {
		my $buf;
		foreach(<R>) {
			$buf.=$_;
		}
		print Jcode->new($buf)->sjis;
		close(R);
	}
	exit;
}
if($ENV{QUERY_STRING} eq "text") {
	print <<EOM;
Content-type: text/plain

EOM
	my $header="■ 東北電力電気速報 ■\n";
	print Jcode->new($header)->sjis;
	if(open(R,"cache/sokuhou.txt")) {
		my $buf;
		foreach(<R>) {
			$buf.=$_;
		}
		print Jcode->new($buf)->sjis;
		close(R);
	}
	exit;
}

if($ENV{QUERY_STRING} eq "html") {
	print <<EOM;
Content-type: text/plain

EOM
	if(open(R,"cache/sokuhou.html")) {
		my $buf;
		foreach(<R>) {
			$buf.=$_;
		}
		print Jcode->new($buf)->sjis;
		close(R);
	}
	exit;
}

if($ENV{QUERY_STRING} eq "mhtml") {
	print <<EOM;
Content-type: text/plain

EOM
	if(open(R,"cache/sokuhou.mhtml")) {
		my $buf;
		foreach(<R>) {
			$buf.=$_;
		}
		print Jcode->new($buf)->sjis;
		close(R);
	}
	exit;
}

if($ENV{QUERY_STRING} eq "twitter") {
	print <<EOM;
Content-type: text/plain

EOM
	if(open(R,"cache/sokuhou.twitter")) {
		my $buf;
		foreach(<R>) {
			$buf.=$_;
		}
		print Jcode->new($buf)->sjis;
		close(R);
	}
	exit;
}

my $rssdate;
my $rssupdate;
my $rss;
my $mitoushi;

if($ENV{QUERY_STRING} eq "rss") {
	&gzip_compress("Content-type: text/html; charset=utf-8\nCache-Control: max-age=0\nExpires: Mon, 26, Jul 1997 05:00:00 GMT");
	if(open(R,"cache/sokuhou.xml")) {
		foreach(<R>) {
			print $_;
		}
		close(R);
	}
	close(STDOUT);
	exit;
}

my $nojapaneseflg=0;
#$nojapaneseflg=1 if($ENV{HTTP_ACCEPT_LANGUAGE}!~/ja/);
my $mobileflg=0;
$mobileflg=1 if($ENV{HTTP_USER_AGENT}=~/DoCoMo|UP\.Browser|KDDI|SoftBank|Voda[F|f]one|J\-PHONE|DDIPOCKET|WILLCOM|iPod|PDA|Mobile|Semulator/);

$mobileflg=0 if($ENV{QUERY_STRING}=~'p');
$mobileflg=1 if($ENV{QUERY_STRING}=~'m');
$nojapaneseflg=0 if($ENV{QUERY_STRING}=~'j');
#$nojapaneseflg=1 if($ENV{QUERY_STRING}=~'e');

my $query=new CGI;

my $mode=$query->param('m');
my $qr=$query->param('qr');

if($mode eq 'qr') {
	my $string=$query->param('str');
	&make_qrcode($string,$qr);
	exit;
}

my $file;
my @data;
my $hash;
my $mailfile;
my $subject;
my %db;
my $english_file="english.html";
my $english_mobile_file="english_mobile.html";
my $mobile_file="mobile.html";
my $pc_file="pc.html";
my $mobile_update_file="mobile_update.html";
my $mobile_setuden_file="mobile_setuden.html";
my $mobile_mailsample_file="mobile_mailsample.html";
my $mobile_twitter_file="mobile_twitter.html";
my $mobile_announce_file="mobile_announcement.html";

my $mail=$query->param('mail');
my $err;
if($mail ne '') {
	if(&mailchk($mail)) {
		my $mode;
		my $regunreg=$query->param('m');
		my $regist=$query->param('regist');
		my $unregist=$query->param('unregist');
		if($regist ne '' || $regunreg eq "regist") {
			$mode="regist";
		} elsif($unregist ne '' || $regunreg eq "unregist") {
			$mode="unregist";
		} else {
			$err="リクエストが異常です。";
			$file="error.html";
		}
		if($file eq '' && $mode eq "regist") {
			&open_db;
			@data=split(/\n/,$::database{$mail});
			foreach(@data) {
				my ($name,$value)=split(/=/,$_);
				$db{$name}=$value;
			}
			if($db{"registed"} eq 1) {
				&close_db;
				$file="error.html";
				$err="このメールアドレスは登録されています。";
			} elsif($::database{$mail} eq '') {
				$file="regist.html";
				$hash=md5_hex($mail . time . $ENV{REMOTE_ADDR});
				$mailfile="registconfirmmail.txt";
				$subject="でんき予報メール仮登録のお知らせ";
				$::database{$mail}=<<EOM;
mail=$mail
confirm=1
hash=$hash
EOM
				&close_db;
			}
		} elsif($file eq '' && $mode eq "unregist") {
			&open_db;
			@data=split(/\n/,$::database{$mail});
			foreach(@data) {
				my ($name,$value)=split(/=/,$_);
				$db{$name}=$value;
			}
			if($db{"confirm"}  eq 1) {
				$file="confirmnow.html";
			} elsif($db{"registed"} eq 1) {
				$file="unregist.html";
				$mailfile="unregistconfirmmail.txt";
				$subject="でんき予報メール仮登録解除のお知らせ";
				$::database{$mail}=<<EOM;
mail=$db{mail}
registed=1
hash=$db{hash}
EOM
				$hash=$db{hash};
			} else {
				$err="登録されていないメールアドレスです。";
				$file="error.html";
			}
			&close_db;
		}
	} else {
		$err="メールアドレスが正しくありません。";
	}
}

my $qhash=$query->param('x');
my $mode=$query->param('mode');

if($mode eq "confirm" && $qhash ne '') {
	&open_db;
	foreach my $email(keys %::database) {
		@data=split(/\n/,$::database{$email});
		foreach(@data) {
			my ($name,$value)=split(/=/,$_);
			$db{$name}=$value;
		}
		if($db{"confirm"} eq 1 && $db{"hash"} eq $qhash && !defined($db{"registed"})) {
			$::database{$email}=<<EOM;
mail=$db{mail}
registed=1
hash=$db{hash}
EOM
			$mail=$db{mail};
			$file="regist_confirm.html";
			$mailfile="registmail.txt";
			$subject="でんき予報メール登録のお知らせ";
			&close_db;
			Nana::Mail::toadmin("Add",$mail,$adminmail);
 			last;
		}
		foreach (keys %db) {
			delete $db{$_};
		}
	}
	if($mail eq '') {
		$err="メールアドレスが正しくないか、メールに添付されたアドレスにきちんとアクセスできていません。";
		$file="error.html";
	}
}

if($mode eq "unregistconfirm" && $qhash ne '') {
	&open_db;
	foreach my $email(keys %::database) {
		@data=split(/\n/,$::database{$email});
		foreach(@data) {
			my ($name,$value)=split(/=/,$_);
			$db{$name}=$value;
		}
		if($db{"registed"} eq 1 && $db{"hash"} eq $qhash) {
			$mail=$db{mail};
			delete $::database{$mail};
			$file="unregist_confirm.html";
			$mailfile="unregist.txt";
			$subject="でんき予報メール登録解除完了のお知らせ";
			&close_db;
			Nana::Mail::toadmin("Del",$mail,$adminmail);
			last;
		}
	}
	if($mail eq '') {
		$err="メールアドレスが正しくないか、メールに添付されたアドレスにきちんとアクセスできていません。";
		$file="error.html";
	}
}

&gzip_compress("Content-type: text/html; charset=utf-8\nCache-Control: max-age=0\nExpires: Mon, 26, Jul 1997 05:00:00 GMT");

my $kanaflg=0;
my $updatetitle;
if($file eq '') {
	if($nojapaneseflg) {
		if($mobileflg) {
			$file=$english_mobile_file;
		} else {
			$file=$english_file;
		}
	} elsif($ENV{QUERY_STRING}=~/^mju/) {
		$file=$mobile_update_file;
		$kanaflg=1;
	} elsif($ENV{QUERY_STRING}=~/^mjs/) {
		$file=$mobile_setuden_file;
		$kanaflg=1;
	} elsif($ENV{QUERY_STRING}=~/^mja/) {
		$file=$mobile_announce_file;
		$kanaflg=1;
	} elsif($ENV{QUERY_STRING}=~/^mjt/) {
		$file=$mobile_twitter_file;
		$kanaflg=1;
	} elsif($ENV{QUERY_STRING}=~/^mjm/) {
		$file=$mobile_mailsample_file;
		$kanaflg=1;
	} elsif($mobileflg) {
		$file=$mobile_file;
	} else {
		$file=$pc_file;
	}
}

my $logdates=&logdates;

if(open(R,"<:utf8","$file")) {
	foreach(<R>) {
		$body.=&convert($_);
	}
	close(R);
} else {
	$body="$file not found. sorry.";
}
if($kanaflg) {
	$body=&z2h($body);
}
binmode(STDOUT, ":utf8");
print "$body$debug\n";
close(STDOUT);

if($mailfile ne '') {
	$body='';
	Nana::Mail::getremotehost;
	if(open(R,"<:utf8","$mailfile")) {
		foreach(<R>) {
			$body.=&convert($_);
		}
		close(R);
		Nana::Mail::send(
			to=>$mail, to_name=> $mail . "様",
			from=>$adminmail, from_name=>$adminname,
			subject=>$subject,data=>$body);
	}
}
exit;

sub logdates {
	my $buf;
	my @tmp;
	my @logdates;
	if(!opendir(DIR,$cache)) {
		print <<EOM;
Content-type: text/plain

$cache not found
EOM
	}
	while(my $file=readdir(DIR)) {
		if($file=~/usage\-(\d+)-(\d+)-(\d+)\.log$/) {
			push(@tmp,"$1-$2-$3");
		}
	}
	closedir(DIR);
	@tmp=reverse sort @tmp;
	foreach(@tmp) {
		my($year,$mon,$day)=split(/-/,$_);
my $date=&date("Y-m-d");
my $hour=&date("H");
		my $selected4="$date" eq $_ && $hour >= 18 ? " selected" : '';
		my $selected3="$date" eq $_ && $hour >= 12 ? " selected" : '';
		my $selected2="$date" eq $_ && $hour >= 6 ? " selected" : ''; 
		my $selected1="$date" eq $_ && $hour >= 0 ? " selected" : '';

		$buf.=sprintf("<option value=\"%04d-%02d-%02d-00\"$selected1>%04d年%02d月%02d日 0時～5時台</option>"
			, $year,$mon, $day, $year,$mon, $day);
		$buf.=sprintf("<option value=\"%04d-%02d-%02d-06\"$selected2>%04d年%02d月%02d日 6時～11時台</option>"
			, $year,$mon, $day, $year,$mon, $day);
		$buf.=sprintf("<option value=\"%04d-%02d-%02d-12\"$selected3>%04d年%02d月%02d日 12時～17時台</option>"
			, $year,$mon, $day, $year,$mon, $day);
		$buf.=sprintf("<option value=\"%04d-%02d-%02d-18\"$selected4>%04d年%02d月%02d日 18時～23時台</option>"
			, $year,$mon, $day, $year,$mon, $day);

	}
	return $buf;
}

sub convert {
	my($text)=shift;
	if(/\@\@/) {
		$text=~s/\@\@VER\@\@/$VER/;
		$text=~s/\@\@ERR\@\@/$err/;
		$text=~s/\@\@INCLUDE\=\"(.+)\"\@\@/@{[&include($1)]}/;
		$text=~s/\@\@INCLUDESCRIPT\=\"(.+)\"\@\@/@{[&include($1,1)]}/;
		$text=~s/\@\@QRCODEEN\@\@/@{[&qrcode_link_en]}/;
		$text=~s/\@\@QRCODEJP\@\@/@{[&qrcode_link]}/;
		$text=~s/\@\@DATAUPDATE\@\@/$data_update/;
		$text=~s/\@\@ENGINEUPDATE\@\@/$engine_update/;
		$text=~s/\@\@TCLOCKUPDATE\@\@/$tclock_update/;
		$text=~s/\@\@TARBALL\@\@/$tarball/;
		$text=~s/\@\@MAIL\@\@/$mail/;
		$text=~s/\@\@HASH\@\@/$hash/;
		$text=~s/\@\@URL\@\@/$basehost$basepath/;
		$text=~s/\@\@REMOTE_ADDR\@\@/$ENV{REMOTE_ADDR}/;
		$text=~s/\@\@REMOTE_HOST\@\@/$ENV{REMOTE_HOST}/;
		$text=~s/\@\@DATETIME\@\@/@{[&date($date_format)]}/;
		$text=~s/\@\@RSS\@\@/$rss/;
		$text=~s/\@\@RSSDATE\@\@/$rssdate/;
		$text=~s/\@\@RSSUPDATE\@\@/$rssupdate/;
		$text=~s/\@\@MITOUSHI\@\@/$mitoushi/;
		$text=~s/\@\@DATES\@\@/$logdates/;
	}
	return $text;
}

sub include {
	my ($file,$mode)=@_;
	my $body="";
	if(open(I,"<:utf8","$file")) {
		my $div=$file;
		$div=~s/\.//g;
		$body.=<<EOM if($mode ne 1);
<div id="$div">
EOM
		foreach(<I>) {
			$body.=$_;
		}
		close(I);
		$body.="</div>\n" if($mode ne 1);
	}
	$body;
}

sub qrcode_link {
	my $string=&enc("$basehost$basepath");
	if(&load_module("GD") && &load_module("GD::Barcode")) {
		return <<FIN;
携帯へURLを送るには、こちらのQRコードをご利用下さい。<br />
<img alt="QRCode" src="$basehost$basepath/?m=qr\&amp;str=$string" />

FIN
	}
	'';
}

sub twitter_proxy {
	my $query=new CGI;
	my $rpp=$query->param('rpp');
	my $q=&enc($query->param('q'));
	my $near=$query->param('near');
	my $within=$query->param('widhin');
	my $units=$query->param('units');
	my $since_id=$query->param('since_id');
	my $callback=$query->param('callback');

	my $searchurl="http://search.twitter.com/search.json";
	$searchurl.="?rpp=$rpp&callback=$callback&q=$q";
	my ($status,$stream)=&httpcl($searchurl);

	if($status eq 0) {
		&gzip_compress("Content-type: application/json");
		print $stream;
	} else {
		print "Content-type: text/plain\n\n";
		print "Cant get '$searchurl'\n";
	}
	exit;
}
